/*
 * pdswave.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */


#include <pds/pdswave.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsWave                                                ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsWave *pds_wave_new(PdsDsNatural Nel)
 *  \brief Crea una estructura de tipo PdsWave de Nel elementos.
 * 
 *  Todos los elementos son rellenados con ceros.
 *  \param[in] Nel Es el número de elementos por ciclo.
 *  \return Un puntero a una estructura de tipo PdsWave.
 *  \ingroup PdsWaveGroup
 */
PdsWave *pds_wave_new(PdsDsNatural Nel)
{
	PdsWave *Wave=NULL;

	if(Nel==0)	return NULL;

	Wave=(PdsWave *)calloc(1,sizeof(PdsWave));
	if(Wave==NULL) {return NULL;}

	Wave->Nel=Nel;
	Wave->V=(PdsDsReal *)calloc(Wave->Nel,sizeof(PdsDsReal));
	if(Wave->V==NULL) 
	{
		free(Wave);
		return NULL;
	}

	Wave->ID=0;
	Wave->x=0;

	return Wave;
}


/** \fn PdsWave *pds_wave_new_vector(const PdsVector *VecSrc)
 *  \brief Crea una estructura de tipo PdsWave usando como fuente el vector VecSrc.
 *  \param[in] VecSrc Es el vector que se usará como fuente.
 *  \return Un puntero a la onda de tipo PdsWave.
 *  \ingroup PdsWaveGroup
 */
PdsWave *pds_wave_new_vector(const PdsVector *VecSrc)
{
	PdsWave *Wave=NULL;
	PdsRaNatural i;

	if(VecSrc==NULL)	return NULL;

	Wave=(PdsWave *)calloc(1,sizeof(PdsWave));
	if(Wave==NULL) {return NULL;}

	Wave->Nel=VecSrc->Nel;
	Wave->V=(PdsDsReal *)calloc(Wave->Nel,sizeof(PdsDsReal));
	if(Wave->V==NULL) 
	{
		free(Wave);
		return NULL;
	}
	else
	{
		for(i=0;i<Wave->Nel;i++)	Wave->V[i]=VecSrc->V[i];
	}

	Wave->ID=0;
	Wave->x=0;

	return Wave;
}


/** \fn int pds_wave_set_data(PdsWave *Wave, PdsDsNatural x, PdsDsReal m)
 *  \brief Escribe el valor m en la posición (x) del generador Wave.
 *  (x) inicia con (0).
 *  \param[in,out] Wave El generador a escribir.
 *  \param[in] x Posición x, el primer valor de x es cero.
 *  \param[in] m El valor en la posición (x), en caso de error por fuera de rango 
 *  (x) entonces no hace nada y no se considera como error.
 *  \return TRUE si todo fue bien o FALSE si no (ej: Vector==NULL). 
 *  \ingroup PdsWaveGroup
 */
int pds_wave_set_data(PdsWave *Wave, PdsDsNatural x, PdsDsReal m)
{
	if(Wave==NULL)	return FALSE;

	if((x>=0)&&(x<Wave->Nel))	Wave->V[x]=m;

	return TRUE;
}


/** \fn int pds_wave_get_value(PdsWave *WAVE,PdsDsReal *y)
 *  \brief Obtiene un valor en la salida e itera el generador de ondas.
 *  \param[in,out] WAVE La estructura de PdsWave a consultar.
 *  \param[out] y En esta variable se carga el  nuevo valor a la salida de generador.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo WAVE==NULL.
 *  \ingroup PdsWaveGroup
 */
int pds_wave_get_value(PdsWave *WAVE,PdsDsReal *y)
{
	if(WAVE==NULL)	return FALSE;

	*y	=WAVE->V[WAVE->ID];
	WAVE->x	=WAVE->V[WAVE->ID];

	WAVE->ID=WAVE->ID+1;
	if(WAVE->ID==WAVE->Nel)	WAVE->ID=0;

	return TRUE;
}


/** \fn int pds_wave_get_last_value(PdsWave *WAVE,PdsDsReal *y)
 *  \brief Obtiene el último valor a la salida del generador de onda.
 *  \param[in,out] WAVE La estructura de PdsWave a consultar.
 *  \param[out] y En esta variable se carga el último valor a la salida de generador.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo WAVE==NULL.
 *  \ingroup PdsWaveGroup
 */
int pds_wave_get_last_value(PdsWave *WAVE,PdsDsReal *y)
{
	if(WAVE==NULL)	return FALSE;

	*y  =WAVE->x;

	return TRUE;
}


/** \fn int pds_wave_get_vector(PdsWave *WAVE,PdsVector *Vector)
 *  \brief Carga un vector con valores obtenidos en la salida del 
 *  generador de onda. Realiza una iteración para cada elemento del vector.
 *  \param[in,out] WAVE La estructura de PdsWave a consultar.
 *  \param[out] Vector En este vector se cargará los valores a la salida de generador.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo WAVE==NULL, Vector==NULL.
 *  \ingroup PdsWaveGroup
 */
int pds_wave_get_vector(PdsWave *WAVE,PdsVector *Vector)
{
	PdsRaNatural i;
	if(WAVE==NULL)		return FALSE;
	if(Vector==NULL)	return FALSE;

	for(i=0;i<Vector->Nel;i++)
	{
		Vector->V[i]=WAVE->V[WAVE->ID];

		WAVE->ID=WAVE->ID+1;
		if(WAVE->ID==WAVE->Nel)	WAVE->ID=0;
	}
	WAVE->x=Vector->V[Vector->Nel-1];

	return TRUE;
}


/** \fn void pds_wave_free(PdsWave *WAVE)
 *  \brief Libera una estructura de tipo puntero PdsWave.
 *  \param[in,out] WAVE La estructura de PdsWave a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsWaveGroup
 */
void pds_wave_free(PdsWave *WAVE)
{
	if(WAVE!=NULL)
	{
		free(WAVE->V);
		free(WAVE);
	}
}


/** \fn void pds_wave_destroy(PdsWave **WAVE)
 *  \brief Libera una estructura de tipo puntero PdsWave, y lo carga con NULL.
 *  \param[in,out] WAVE La estructura de PdsWave a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsWaveGroup
 */
void pds_wave_destroy(PdsWave **WAVE)
{
	if((*WAVE)!=NULL)
	{
		free((*WAVE)->V);
		free(*WAVE);
		(*WAVE)=NULL;
	}
}


